@@ -6,9 +6,8 @@ from securetrading.test import abstract_test
 import time
 import hashlib
 import os
-import platform
-import sys
 import argparse
+import sys
 
 
 def get_args():
@@ -30,6 +29,12 @@ def get_args():
                              help="Sets the sleep time for parent transactions\
  to be fully processed in seconds. Default: %(default)d")
 
+    args_parser.add_argument("--datacenterurl", action="store",
+                             dest="datacenterurl",
+                             default="https://webservices.securetrading.net",
+                             help="Sets the datacenter URL to use to process the\
+ transactions. Default: %(default)s")
+
     (passed_args, unittest_args) = args_parser.parse_known_args()
     return passed_args, unittest_args
 
@@ -61,6 +66,7 @@ class Module_Test_Api(abstract_test.TestCase):
 
         username = passed_args.username
         password = passed_args.password
+        datacenterurl = passed_args.datacenterurl
         ssl_cert_file = passed_args.overridecacerts
         base_path = self.get_package_path()
 
@@ -69,12 +75,14 @@ class Module_Test_Api(abstract_test.TestCase):
         st_config.username = username
         st_config.password = password
         st_config.ssl_certificate_file = ssl_cert_file
+        st_config.datacenterurl = datacenterurl
 
         # st_api2 is intentionally using bad credentials.
         st_config2 = securetrading.Config()
         st_config2.username = "UNKNOWN"
         st_config2.password = "PASS"
         st_config2.ssl_certificate_file = ssl_cert_file
+        st_config2.datacenterurl = datacenterurl
 
         # st_api3 is intentionally using a corrupt cacerts file.
         st_config3 = securetrading.Config()
@@ -82,6 +90,7 @@ class Module_Test_Api(abstract_test.TestCase):
         st_config3.password = password
         st_config3.ssl_certificate_file = os.path.join(base_path,
                                                        "test/badcacert.pem")
+        st_config3.datacenterurl = datacenterurl
 
         # st_api4 is intentionally using a cacerts file that is not our ca.
         st_config4 = securetrading.Config()
@@ -89,6 +98,7 @@ class Module_Test_Api(abstract_test.TestCase):
         st_config4.password = password
         st_config4.ssl_certificate_file = os.path.join(base_path,
                                                        "test/testcacert.pem")
+        st_config4.datacenterurl = datacenterurl
 
         # st_api valid credentials fr setup
         st_config_fr = securetrading.Config()
@@ -96,6 +106,23 @@ class Module_Test_Api(abstract_test.TestCase):
         st_config_fr.password = password
         st_config_fr.ssl_certificate_file = ssl_cert_file
         st_config_fr.locale = "fr_fr"
+        st_config_fr.datacenterurl = datacenterurl
+
+        # st_api valid credentials de setup
+        st_config_de = securetrading.Config()
+        st_config_de.username = username
+        st_config_de.password = password
+        st_config_de.ssl_certificate_file = ssl_cert_file
+        st_config_de.locale = "de_de"
+        st_config_de.datacenterurl = datacenterurl
+
+        # st_api valid credentials with Content-type response header
+        st_config_content_type = securetrading.Config()
+        st_config_content_type.username = username
+        st_config_content_type.password = password
+        st_config_content_type.ssl_certificate_file = ssl_cert_file
+        st_config_content_type.http_response_headers = ["content-type"]
+        st_config_content_type.datacenterurl = datacenterurl
 
         # initialise all the st_api objects with their respective config
         self.st_api = securetrading.Api(st_config)
@@ -103,6 +130,8 @@ class Module_Test_Api(abstract_test.TestCase):
         self.st_api3 = securetrading.Api(st_config3)
         self.st_api4 = securetrading.Api(st_config4)
         self.st_api_fr = securetrading.Api(st_config_fr)
+        self.st_api_de = securetrading.Api(st_config_de)
+        self.st_api_content_type = securetrading.Api(st_config_content_type)
 
         if self.PARENT_RESPONSES is None:
             # Singleton so that it doenst get set always
@@ -131,6 +160,10 @@ class Module_Test_Api(abstract_test.TestCase):
                              "AUTH",
                              {"settlestatus": "100",
                               "paymenttypedescription": "PAYPAL"}),
+                            ("settled_auth_refund",
+                             "AUTH",
+                             {"settlestatus": "100",
+                              "paymenttypedescription": "PAYPAL"}),
                             ("cachetoken",
                              "CACHETOKENISE",
                              {"pan": "4111111111111111",
@@ -142,7 +175,7 @@ class Module_Test_Api(abstract_test.TestCase):
                             ]
 
         for parent_key, requesttypdescription, updates in parents_required:
-            if parent_key == "settled_auth":
+            if parent_key.startswith("settled_auth"):
                 p_ref = parent_responses["order"]["transactionreference"]
                 updates["parenttransactionreference"] = p_ref
                 paypaltoken = parent_responses["order"]["paypaltoken"]
@@ -280,7 +313,7 @@ class Module_Test_Api(abstract_test.TestCase):
         exp_raw_resp = [exp_resp_data]
         self.validate(st_response["responses"], exp_raw_resp)
 
-    def test_auth_fr(self):
+    def test_auth_content_type(self):
         extra_updates = {"pan": "4111111111111111",
                          "expirymonth": "11",
                          "expiryyear": "2031",
@@ -291,13 +324,56 @@ class Module_Test_Api(abstract_test.TestCase):
         data = self.get_request_values("AUTH", extra_updates=extra_updates)
 
         st_request = self._get_st_request(data)
-        st_response = self.st_api_fr.process(st_request)
+        st_response = self.st_api_content_type.process(st_request)
 
         exp_resp_data = {"errorcode": "0",
-                         "errormessage": "D'accord",
+                         "errormessage": "Ok",
                          "acquirerresponsecode": "00",
                          }
         exp_raw_resp = [exp_resp_data]
+        exp_raw_headers = {"Content-Type": "application/json"}
+        self.validate(st_response["responses"], exp_raw_resp)
+        self.validate(st_response["headers"], exp_raw_headers)
+
+    def test_auth_decline_fr(self):
+        extra_updates = {"pan": "4242424242424242",
+                         "expirymonth": "11",
+                         "expiryyear": "2031",
+                         "securitycode": "123",
+                         "paymenttypedescription": "VISA",
+                         }
+        data = self.get_request_values("AUTH", extra_updates=extra_updates)
+
+        st_request = self._get_st_request(data)
+        st_response = self.st_api_fr.process(st_request)
+
+        exp_resp_data = {"errorcode": "70000",
+                         "errormessage": "Refuser",
+                         "authcode": "DECLINED",
+                         }
+
+        exp_raw_resp = [exp_resp_data]
+        self.validate(st_response["responses"], exp_raw_resp)
+
+    def test_auth_decline_de(self):
+        extra_updates = {"pan": "4242424242424242",
+                         "expirymonth": "11",
+                         "expiryyear": "2031",
+                         "securitycode": "123",
+                         "paymenttypedescription": "VISA",
+                         }
+
+        data = self.get_request_values("AUTH", extra_updates=extra_updates)
+
+        st_request = self._get_st_request(data)
+        st_response = self.st_api_de.process(st_request)
+
+        exp_resp_data = {"errorcode": "70000",
+                         "errormessage": "ablehnen",
+                         "authcode": "DECLINED",
+                         }
+
+        exp_raw_resp = [exp_resp_data]
         self.validate(st_response["responses"], exp_raw_resp)
 
     def test_auth_from_token(self):
@@ -797,7 +873,8 @@ class Module_Test_Api(abstract_test.TestCase):
         self.validate(st_response["responses"], exp_raw_resp)
 
     def test_transactionquery(self):
-        p_ref = self.PARENT_RESPONSES["pending_auth"]["transactionreference"]
+        p_ref = (self.PARENT_RESPONSES["pending_auth"]
+                 ["transactionreference"])
         data = {"requesttypedescriptions": ["TRANSACTIONQUERY"],
                 "filter": {"transactionreference": [{"value": p_ref}],
                            }
@@ -899,6 +976,103 @@ class Module_Test_Api(abstract_test.TestCase):
 
         self.validate(st_response["responses"], exp_raw_resp)
 
+    def test_separate_transactionupdate_refund_update(self):
+        p_ref = self.PARENT_RESPONSES["pending_auth"]["transactionreference"]
+        rf_extra = {"parenttransactionreference": p_ref,
+                    "baseamount": "15",
+                    }
+
+        rf_data = self.get_request_values("REFUND",
+                                          extra_updates=rf_extra)
+
+        rf_data["requesttypedescription"] = rf_data.pop(
+            "requesttypedescriptions")[0]
+
+        trxup_extra = {"requesttypedescriptions": ["TRANSACTIONUPDATE"],
+                       "filter": {"transactionreference": [{"value": p_ref}],
+                                  },
+                       "updates": {"settlebaseamount": "35",
+                                   },
+                       }
+
+        trxup_data = self.get_request_values("REFUND",
+                                             extra_updates=trxup_extra)
+        trxup_data["requesttypedescription"] = trxup_data.pop(
+            "requesttypedescriptions")[0]
+
+        st_response = self.process_multiple([trxup_data, rf_data])
+
+        exp_raw_resp = [{"errorcode": "0",
+                         "errormessage": "Ok",
+                         "requesttypedescription": "TRANSACTIONUPDATE"
+                         }]
+
+        self.validate(st_response["responses"], exp_raw_resp)
+
+    def test_separate_transactionupdate_refund_refund(self):
+        p_ref = (self.PARENT_RESPONSES["settled_auth_refund"]
+                 ["transactionreference"])
+        rf_extra = {"parenttransactionreference": p_ref,
+                    "baseamount": "20",
+                    }
+
+        rf_data = self.get_request_values("REFUND",
+                                          extra_updates=rf_extra)
+
+        rf_data["requesttypedescription"] = rf_data.pop(
+            "requesttypedescriptions")[0]
+
+        trxup_extra = {"requesttypedescriptions": ["TRANSACTIONUPDATE"],
+                       "filter": {"transactionreference": [{"value": p_ref}],
+                                  },
+                       "updates": {"settlebaseamount": "80",
+                                   },
+                       }
+
+        trxup_data = self.get_request_values("REFUND",
+                                             extra_updates=trxup_extra)
+        trxup_data["requesttypedescription"] = trxup_data.pop(
+            "requesttypedescriptions")[0]
+
+        st_response = self.process_multiple([trxup_data, rf_data])
+
+        exp_raw_resp = [{"errorcode": "60017",
+                         "errormessage": "Transaction not updatable",
+                         "requesttypedescription": "TRANSACTIONUPDATE",
+                         },
+                        {"errorcode": "0",
+                         "errormessage": "Ok",
+                         "requesttypedescription": "REFUND"
+                         }]
+
+        self.validate(st_response["responses"], exp_raw_resp)
+
+    def test_multi_transactionupdate_update_refund(self):
+
+        extra_updates = {"pan": "4111111111111111",
+                         "expirymonth": "11",
+                         "expiryyear": "2031",
+                         "securitycode": "123",
+                         "paymenttypedescription": "VISA",
+                         "baseamount": "100",
+                         "requesttypedescriptions": ["TRANSACTIONUPDATE",
+                                                     "REFUND"]
+                         }
+
+        data = self.get_request_values("TRANSACTIONUPDATE",
+                                       extra_updates=extra_updates)
+        data = self.get_request_values("REFUND",
+                                       extra_updates=data)
+
+        st_response = self.process_single(data)
+
+        exp_raw_resp = [{"errorcode": "30000",
+                         "errormessage": "Invalid field",
+                         "errordata": ["requesttypedescriptions"],
+                         }]
+
+        self.validate(st_response["responses"], exp_raw_resp)
+
 
 if __name__ == "__main__":
     script_name = sys.argv[0]
