@@ -1,12 +1,10 @@
 #!/usr/bin/env python
 from __future__ import unicode_literals
-import sys
 import unittest
 from securetrading.test import abstract_test
 import securetrading
 import securetrading.httpclient as st_httpclient
 import json
-import platform
 
 
 class Test_Api(abstract_test.TestCase):
@@ -39,34 +37,36 @@ class Test_Api(abstract_test.TestCase):
         connecterror = securetrading.ConnectionError
         sendrecverror = securetrading.SendReceiveError
 
-        versioninfo = "Python::{0}::1.0.5::{1}".format(
-            platform.python_version(),
-            platform.platform())
         request1 = self.get_securetrading_request({})
         request4 = self.get_securetrading_request({})
         default_config = self.get_config()
         config1 = self.get_config({"username": "test",
                                    "jsonversion": "2.00",
                                    "datacenterurl": "https://test.com",
+                                   "datacenterpath": '/some/path/',
+                                   "http_response_headers": ["headers"],
                                    })
 
         french_config = self.get_config({"locale": "fr_fr",
                                          })
 
+        german_config = self.get_config({"locale": "de_de",
+                                         })
+
         request2 = self.get_securetrading_request(
-            {"datacenterurl": "https://somewhere.com"})
+            {"datacenterurl": "https://somewhere.com",
+             "datacenterpath": "/some/other/path/"})
 
         request1_str = '{{"requestreference":"{0}",\
 "version":"1.00", "response":[{{"errorcode" : "0"}}]}}'
         request1_str2 = '{{"requestreference":"{0}",\
 "version":"1.00", "response":[]}}'
         request1_dict = {"requestreference": request1["requestreference"],
-                         "versioninfo": versioninfo}
+                         "versioninfo": self.version_info}
 
         request2_str = '{{"requestreference": "{0}",\
 "version": "1.00", "response": [{{"errorcode" : "0"}}]}}'
 
-        lib_version = "python_1.0.5"
         msg = "{0} Maximum time reached whilst trying to connect to {1}\
 ".format(request2["requestreference"], request2["datacenterurl"])
         connection_error = connecterror("7", data=[msg])
@@ -78,7 +78,7 @@ class Test_Api(abstract_test.TestCase):
             {"requestreference": "requestreference"})
 
         request4_dict = {"requestreference": request4["requestreference"],
-                         "versioninfo": versioninfo}
+                         "versioninfo": self.version_info}
 
         request4_str = '{{"requestreference": "{0}",\
 "version": "1.00", "response": [{{"errormessage": "GATEWAYERRMSG",\
@@ -88,10 +88,10 @@ class Test_Api(abstract_test.TestCase):
             jsonMessage = ["Expecting value: line 1 column 21 (char 20)"]
         test_cases = [(request1, default_config,
                        request1_str.format(request1["requestreference"]),
-                       None,
+                       {"Headers": "data"}, None,
                        "https://webservices.securetrading.net/json/",
                        {"alias": "",
-                        "libraryversion": lib_version,
+                        "libraryversion": self.lib_version,
                         "request": [request1_dict],
                         "version": "1.00"},
                        None, None,
@@ -99,13 +99,15 @@ class Test_Api(abstract_test.TestCase):
                         "version": "1.00",
                         "responses": [{"errormessage": "Ok",
                                        "errorcode": "0"
-                                       }]}),
+                                       }]},
+                       None),
                       # config with new values instead of defaults
                       (request1, config1,
                        request1_str.format(request1["requestreference"]),
-                       None, "https://test.com/json/",
+                       {"Headers": "data"}, None,
+                       "https://test.com/some/path/",
                        {"alias": "test",
-                        "libraryversion": lib_version,
+                        "libraryversion": self.lib_version,
                         "request": [request1_dict],
                         "version": "2.00"},
                        None, None,
@@ -113,40 +115,45 @@ class Test_Api(abstract_test.TestCase):
                         "version": "1.00",
                         "responses": [{"errormessage": "Ok",
                                        "errorcode": "0"},
-                                      ]}),
+                                      ]},
+                       {"Headers": "data"}),
                       # config with new values instead of defaults
                       (request1, config1,
                        request1_str2.format(request1["requestreference"]),
-                       None, "https://test.com/json/",
+                       {"Accept": "*/*"}, None, "https://test.com/some/path/",
                        {"alias": "test",
-                        "libraryversion": lib_version,
+                        "libraryversion": self.lib_version,
                         "request": [request1_dict],
                         "version": "2.00"},
                        None, None,
                        {"requestreference": request1["requestreference"],
                         'version': '1.00',
-                        "responses": []}),
+                        "responses": []},
+                       {"Accept": "*/*"}),
                       # Invalid response example
                       (request2, config1,
                        request2_str.format(request2["requestreference"]),
-                       None, "https://somewhere.com/json/",
+                       {"Headers": "data"}, None,
+                       "https://somewhere.com/some/other/path/",
                        {"alias": "test",
-                        "libraryversion": lib_version,
+                        "libraryversion": self.lib_version,
                         "request": [{"requestreference":
                                      request2["requestreference"],
                                      "datacenterurl": "https://somewhere.com",
-                                     "versioninfo": versioninfo,
+                                     "datacenterpath": "/some/other/path/",
+                                     "versioninfo": self.version_info,
                                      }],
                         "version": "2.00"},
                        None, None,
                        {'requestreference': request2["requestreference"],
                         'version': '1.00',
                         'responses': [{"errormessage": "Ok",
-                                       "errorcode": "0"}]}),
+                                       "errorcode": "0"}]},
+                       {"Headers": "data"}),
                       # config with new values and request overrides
                       # datacenterurl
-                      ([], default_config, None, None, None, None, None,
-                       None,
+                      ([], default_config, None, {"Headers": "data"}, None,
+                       None, None, None, None,
                        {'requestreference': '',
                         'responses': [{'errorcode': '10',
                                        'errordata':
@@ -157,11 +164,11 @@ request specified'],
                                        'errormessage':
                                            'Incorrect usage of the \
 Secure Trading API'},
-                                      ]}
-                       ),
+                                      ]},
+                       None),
 
-                      ([], french_config, None, None, None, None, None,
-                       None,
+                      ([], french_config, None, {"Headers": "data"}, None,
+                       None, None, None, None,
                        {'requestreference': '',
                         'responses': [{'errorcode': '10',
                                        'errordata':
@@ -170,12 +177,27 @@ request specified'],
                                        'requesttypedescription': 'ERROR',
                                        'requestreference': '',
                                        'errormessage':
-                                           "Une utilisation incorrecte de \
+                                           "Utilisation incorrecte de \
 l'API Secure Trading"},
-                                      ]}
-                       ),  # French version of the config
-                      (request2, default_config, None, connecterror("7"), None,
-                       None, None, None,
+                                      ]},
+                       None),  # French version of the config
+                      ([], german_config, None, {"Headers": "data"}, None,
+                       None, None, None, None,
+                       {'requestreference': '',
+                        'responses': [{'errorcode': '10',
+                                       'errordata':
+                                           ['Incorrect type of \
+request specified'],
+                                       'requesttypedescription': 'ERROR',
+                                       'requestreference': '',
+                                       'errormessage':
+                                           'Fehlerhafte Verwendung der Secure \
+Trading API'},
+                                      ]},
+                       None),  # German version of the config
+
+                      (request2, default_config, None, {"Headers": "data"},
+                       connecterror("7"), None, None, None, None,
                        {"requestreference": request2["requestreference"],
                         "responses":
                             [{"requesttypedescription": "ERROR",
@@ -183,9 +205,10 @@ l'API Secure Trading"},
                               "errorcode": "7",
                               "errormessage": "An issue occured whilst trying to \
 connect to Secure Trading servers",
-                              "errordata": []},
-                             ]}),
-                      (request2, default_config, None,
+                              "errordata": []}
+                             ]},
+                       None),
+                      (request2, default_config, None, {"Headers": "data"},
                        sendrecverror("4", connection_error), None, None, None,
                        None,
                        {"requestreference": request2["requestreference"],
@@ -197,19 +220,19 @@ connect to Secure Trading servers",
                               "errordata": ['7 {0} Maximum time reached whilst trying to \
 connect to https://somewhere.com'.format(request2["requestreference"])]
                               }]
-                        }
-                       ),
+                        },
+                       None),
                       (request1,
                        default_config,
                        '{"requestreference": \
 "NEWREFERENCE-866-98", "version": "1.00", "response": [{"errorcode" : "0" }]}',
-                       None,
+                       {"Headers": "data"}, None,
                        "https://webservices.securetrading.net/json/",
                        {"alias": "",
-                        "libraryversion": lib_version,
+                        "libraryversion": self.lib_version,
                         "request":
                         [{"requestreference": request1["requestreference"],
-                          "versioninfo":versioninfo}],
+                          "versioninfo":self.version_info}],
                         "version": "1.00"},
                        None, None,
                        {"requestreference": request1["requestreference"],
@@ -221,18 +244,19 @@ connect to https://somewhere.com'.format(request2["requestreference"])]
  please contact Secure Trading',
                               'errordata':
                                   ['Different request reference: sent ({0}) received\
- (NEWREFERENCE-866-98)'.format(request1["requestreference"])]}]}),
+ (NEWREFERENCE-866-98)'.format(request1["requestreference"])]}]},
+                       None),
                       (request1,
                        default_config,
                        '{"requestreference":: \
 "NEWREFERENCE-866-98", "version": "1.00", "response": []}',
-                       None,
+                       {"Headers": "data"}, None,
                        "https://webservices.securetrading.net/json/",
                        {"alias": "",
-                        "libraryversion": lib_version,
+                        "libraryversion": self.lib_version,
                         "request":
                         [{"requestreference": request1["requestreference"],
-                          "versioninfo":versioninfo}],
+                          "versioninfo":self.version_info}],
                         "version": "1.00"},
                        None, None,
                        {"requestreference": request1["requestreference"],
@@ -244,16 +268,16 @@ connect to https://somewhere.com'.format(request2["requestreference"])]
                               'errordata': jsonMessage,
                               }
                              ]
-                        }
-                       ),
+                        },
+                       None),
                       ({"requestreference": "myref"}, default_config,
                        request1_str.format("myref"),
-                       None,
+                       {"Headers": "data"}, None,
                        "https://webservices.securetrading.net/json/",
                        {"alias": "",
-                        "libraryversion": lib_version,
+                        "libraryversion": self.lib_version,
                         "request": [{"requestreference": "myref",
-                                     "versioninfo": versioninfo,
+                                     "versioninfo": self.version_info,
                                      }],
                         "version": "1.00"},
                        None, None,
@@ -261,13 +285,14 @@ connect to https://somewhere.com'.format(request2["requestreference"])]
                         "version": "1.00",
                         "responses": [{"errormessage": "Ok",
                                        "errorcode": "0"
-                                       }]}),
+                                       }]},
+                       None),
                       (request4, default_config,
                        request4_str.format(request4["requestreference"]),
-                       None,
+                       {"Headers": "data"}, None,
                        "https://webservices.securetrading.net/json/",
                        {"alias": "",
-                        "libraryversion": lib_version,
+                        "libraryversion": self.lib_version,
                         "request": [request4_dict],
                         "version": "1.00"},
                        None, None,
@@ -275,16 +300,17 @@ connect to https://somewhere.com'.format(request2["requestreference"])]
                         "version": "1.00",
                         "responses": [{"errormessage": "GATEWAYERRMSG",
                                        "errorcode": "99"
-                                       }]}),
+                                       }]},
+                       None),
                       ]
 
         http_main = st_httpclient.GenericHTTPClient._main
         try:
-            for (request, config, http_result, http_raises,
+            for (request, config, http_result, http_headers, http_raises,
                  http_url, http_json_dict, exp_raises, exp_message,
-                 exp_result) in test_cases:
+                 exp_result, exp_headers) in test_cases:
                 st_httpclient.GenericHTTPClient._main = self.mock_method(
-                    result=http_result,
+                    result=(http_result, http_headers),
                     exception=http_raises)
                 api = securetrading.Api(config=config)
                 if exp_raises is None:
@@ -295,6 +321,7 @@ connect to https://somewhere.com'.format(request2["requestreference"])]
                         actual_json_dict = json.loads(sent_to_mock[0][0][2])
                         self.assertEqual(actual_url, http_url)
                         self.assertEqual(actual_json_dict, http_json_dict)
+                    self.assertEqual(actual.pop("headers", None), exp_headers)
                     self.assertEqual(actual, exp_result)
                 else:
                     self.assertRaisesRegexp(
